# module to ingest data to a Wendelin through HTTP endpoint
# Copyright (C) 2015  Nexedi SA and Contributors.
#                     Kirill Smelkov <kirr@nexedi.com>
#
#   This program is free software: you can Use, Study, Modify and Redistribute
#   it under the terms of the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


require 'net/http'
require 'openssl'

# class representing a Wendelin client
class WendelinClient

  # `streamtool_uri` - URI pointing to portal_input_data_stream "mountpoint"
  # `credentials`    # {'user' => _, 'password' => _}     TODO change to certificate
  # `log` - logger to use
  def initialize(streamtool_uri, credentials, log, reference)
    @streamtool_uri = streamtool_uri
    @credentials    = credentials
    @log            = log
    @reference = reference

  end

  # start request in an independent function to keep the connection open
  def start_request

    uri = URI("#{@streamtool_uri}/ingest?reference=#{@reference}")
    @http = Net::HTTP.start(uri.hostname, uri.port,
      :use_ssl => (uri.scheme == 'https'),
      :verify_mode  => OpenSSL::SSL::VERIFY_NONE,
      :ssl_timeout  => 60,
      :open_timeout => 60,
      :read_timeout => 60,
      :keep_alive_timeout => 60,)
      @request = Net::HTTP::Post.new(uri)

  end

  # ingest `data_chunk` to a stream referenced as `reference`
  def ingest(data_chunk)

    # When using 'application/x-www-form-urlencoded', Ruby encodes with regex
    # and it is far too slow. Such POST is legit:
    # https://stackoverflow.com/a/14710450
    @request ||= start_request     # call start_request if request is undefined
    @request.body = data_chunk
    @request.content_type = 'application/octet-stream'
    if @credentials.has_key?('user')
      @request.basic_auth @credentials['user'], @credentials['password']
    end

      @log.on_trace do
          @log.trace '>>> REQUEST'
          @log.trace "method\t=> #{@request.method}"
          @log.trace "path\t=> #{@request.path}"
          @log.trace "uri\t=> #{@request.uri}"
          @log.trace "body\t=> #{@request.body}"
          @log.trace "body_stream\t=> #{@request.body_stream}"
          @request.each {|h| @log.trace "#{h}:\t#{@request[h]}"}
          @log.trace
      end

      begin
          res = @http.request(@request) # Net::HTTPResponse object
      end

      rescue
          # some http/ssl/other connection error
          @log.warn "HTTP ERROR:"
          raise

      else
          @log.on_trace do
              @log.trace '>>> RESPONSE'
              res.each {|h| @log.trace "#{h}:\t#{res[h]}"}
              @log.trace "code\t=> #{res.code}"
              @log.trace "msg\t=> #{res.message}"
              @log.trace "class\t=> #{res.class}"
              @log.trace "body:", res.body
          end

          if res.kind_of?(Net::HTTPSuccess)       # res.code is 2XX
              #@log.info "ingested ok"
          else
              @log.warn "FAIL:"
              res.value
          end
      end

end
